/*
DarkEnergy

Copyright (C) 2012 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU.

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/


/**
 * Algorithme de simulation pour SimuApplet.
 * Ce programme calcule l'age de l'univers et l'évolution du facteur d'echelle "a" en fonction du temps "t"
 * Les modeles d'Univers consideres sont ceux de Friedmann-Lemaitre-Robertson-Walker (AgeUnivers)
 */

new AgeUnivers();

function AgeUnivers() {
    this.applet = new SimuApplet(this);
    this.applet.appletDoc("age_univers_doc.xml");
    
	this.mpc = 3.08567758074e22;
	this.gyr = 3.1688087814e-17,g=6.673231e-11;
    this.g = 6.673231e-11;
	this.c = 2.99792458e8;
	this.twopi = 6.283185307179586;
	this.a0 = 1.;
	this.eres = null; // double[];
	this.eres2 = null; // double[];
	this.h0 = 0.;
    this.Omega_m0 = 0.;
    this.Omega_rad0 = 0.;
    this.Omega_l0 = 0.;
    this.Omega_k0 = 0.;
    this.T0 = 0.;
    this.temp = 0.;
	this.rho_m = 0.;
    this.rho_rad = 0.;
    this.rho_lam = 0.;
	this.stop = 1e-5;
	this.da = 0.;
	this.a = 0.;
    this.t = 0.;
    this.integral = 0.;
    this.age = 0.;
    //this.courbure = 0.;
	this.tmp = 0.;
	this.nobb = false,
    this.nobc = false;
    this.failed = false;
    this.half = false; 
}

AgeUnivers.prototype.initCalculLive = function(lin) {
    this.h0 = lin.lireDouble("h0");
    this.Omega_m0 = lin.lireDouble("omega-m0");
    if (this.Omega_m0 == 0)
        throw new Error('Omega_m0 doit être > 0'); // sinon on essaye d'afficher rho_m qui vaut 0 en log
    this.Omega_l0 = lin.lireDouble("omega-l0");
    if (this.Omega_l0 == 0)
        throw new Error('Omega_l0 doit être > 0'); // sinon on essaye d'afficher rho_lam qui vaut 0 en log
    this.Omega_rad0 = lin.lireDouble("omega-rad0");
    this.Omega_rad0 = 4.1534e-5 / this.h0 / this.h0 * 100 * 100 * Math.pow((this.Omega_rad0 / 2.725), 4);

    this.a = this.a0;
    this.t = 0;
    this.da = 1e-2;

    this.h0 = this.h0 * 1.e3 / this.mpc;
    this.Omega_k0 = this.Omega_rad0 + this.Omega_m0 + this.Omega_l0 - 1.;

    if (this.Omega_m0 <= 0.5) {
        this.tmp = (1 - this.Omega_m0) / this.Omega_m0;
        this.tmp = Math.log(this.tmp + Math.sqrt(this.tmp * this.tmp - 1.)) / 3;
        this.tmp = (Math.exp(this.tmp) + Math.exp(-this.tmp)) / 2;
        this.tmp = 4 * this.Omega_m0 * Math.pow(this.tmp, 3.);
    }

    if (this.Omega_m0 > 0.5) {
        this.tmp = (1 - this.Omega_m0) / this.Omega_m0;
        this.tmp = 4 * this.Omega_m0 * Math.pow(Math.cos(Math.acos(this.tmp) / 3), 3.);
    }

    this.nobb = false;
    if (this.Omega_l0 >= this.tmp) {
        this.nobb = true;
    }

    if (this.Omega_m0 > 1) {
        this.tmp = (1 - this.Omega_m0) / this.Omega_m0;
        this.tmp = 4 * this.Omega_m0 * Math.pow(Math.cos(Math.acos(this.tmp) / 3) + 4 / 3 * Math.PI, 3.);
    }
    if (this.Omega_m0 <= 1){
        this.tmp = 0;
    }
    this.nobc = true;
    if (this.Omega_l0 <= this.tmp) {
        this.nobc = false;
    }


    if (!this.nobc){
        this.a = 1e-3;
        this.half = false;
    }
}

	// Calcul de a à a+da
AgeUnivers.prototype.calculLive = function() {
    var out;
    
    out = new ListeValeurs();

    if (this.nobc) {
        if( (!this.nobb & this.a < 1e-4))
            return null; // fin du calcul

        if (this.nobb & this.failed )
            return null; // fin du calcul

        if (this.a <= 1e-2)
            this.da = 1e-4;

        this.integral = this.rombint(this.hubble, this.a, this.a - this.da, this.stop);
        this.t += this.integral;
        this.a += -this.da;

        this.rho_m = this.Omega_m0 * 3. * this.h0 * this.h0 / 4. / this.twopi / this.g * Math.pow((this.a0 / this.a), 3.);
        this.rho_rad = this.Omega_rad0 * 3. * this.h0 * this.h0 / 4. / this.twopi / this.g * Math.pow((this.a0 / this.a), 4.);
        this.rho_lam = this.Omega_l0 * 3. * this.h0 * this.h0 / 4. / this.twopi / this.g;
        
        out.ajouterDouble('t', this.t);
        out.ajouterDouble('a', this.a);
        out.ajouterDouble('temp', this.T0 / this.a);
        out.ajouterDouble('rho-m', this.rho_m);
        out.ajouterDouble('rho-rad', this.rho_rad);
        out.ajouterDouble('rho-lam', this.rho_lam);
        this.age = Math.floor(10. * this.t) / 10.;
        this.age = -this.age;
        //courbure=Math.floor(100.*Omega_k0)/10.;
        if (Math.abs(this.Omega_k0) < 1E-10){
            out.ajouterString('comment',"Univers plat ; Age de l'Univers : " + this.age + " Milliards d'années ");
        } else if (this.Omega_k0 > 0) {
            out.ajouterString('comment', "Univers elliptique ; Age de l'Univers : " + this.age + " Milliards d'années ");
        } else if (this.Omega_k0 < 0){
            out.ajouterString('comment',"Univers hyperbolique ; Age de l'Univers : " + this.age + " Milliards d'années ");
        }
        return out;
    }

    if (!this.nobc) {
        this.tmp = this.Omega_m0 * Math.pow(this.a0 / this.a, 3.)
            + this.Omega_rad0 * Math.pow(this.a0 / this.a, 4.)
            - this.Omega_k0 * Math.pow(this.a0 / this.a, 2.)
            + this.Omega_l0;

        //	if( tmp<=1e-5 & half)
        if (this.failed && this.half)
            return null; // fin du calcul

        if (this.a <= 1e-2)
            this.da = 1e-4;

        if (this.a > 1e-2)
            this.da = 1e-2;
        this.integral = this.rombint(this.hubble, this.a, this.a + this.da, this.stop);
        if (this.failed && !this.half) {
            this.half = true;
            this.a += -1e-2;
            this.integral = this.rombint(this.hubble, this.a, this.a + this.da, this.stop);
            //System.err.println("tmp "+tmp+" nobc "+nobc);
        }
        if (!this.failed) 
            this.t += this.integral;

        if (!this.half)
            this.a += this.da;
        if (this.half)
            this.a += -this.da;

        this.rho_m = this.Omega_m0 * 3. * this.h0 * this.h0 / 4. / this.twopi / this.g * Math.pow((this.a0 / this.a), 3.);
        this.rho_rad = this.Omega_rad0 * 3. * this.h0 * this.h0 / 4. / this.twopi / this.g * Math.pow((this.a0 / this.a), 4.);
        this.rho_lam = this.Omega_l0 * 3. * this.h0 * this.h0 / 4. / this.twopi / this.g;

        out.ajouterDouble('t', this.t);
        out.ajouterDouble('a', this.a);
        out.ajouterDouble('temp', this.T0 / this.a);
        out.ajouterDouble('rho-m', this.rho_m);
        out.ajouterDouble('rho-rad', this.rho_rad);
        out.ajouterDouble('rho-lam', this.rho_lam);
        this.age = Math.floor(10. * this.t) / 10.;
        if (Math.abs(this.Omega_k0) < 1E-10){
            out.ajouterString('comment', "Univers plat ; Age de l'Univers : " + this.age + " Milliards d'années ");
        } else if (this.Omega_k0 > 0) {
            out.ajouterString('comment', "Univers elliptique ; Age de l'Univers : " + this.age + " Milliards d'années ");
        } else if (this.Omega_k0 < 0) {
            out.ajouterString('comment', "Univers hyperbolique ; Age de l'Univers : " + this.age + " Milliards d'années ");
        }
        return out;
    }
    return null;
}

// pour l'affichage tableau
AgeUnivers.prototype.calcul = function(lin) {
    var out;
    
    out = new ListeValeurs();
    this.initParamIn(lin);      
    
    this.da = 1e-3;
    while (this.a >= 1 / this.eres[1]) {
        this.integral = this.rombint(this.hubble, this.a, this.a - this.da, this.stop);
        this.t += this.integral;
        this.a += -this.da;
    }
    
    this.age = Math.abs(this.t);
    out.ajouterDouble('t', this.age / this.gyr);
    out.ajouterDouble('temp', this.eres[0] * this.T0);
    out.ajouterString('comment', "Aujourd'hui");

    this.t = Math.pow(1. / this.eres[1] / Math.pow(2. * this.h0 * this.h0 * this.Omega_rad0, 0.25) / this.a0, 2.);
    out.ajouterDouble('t', this.t);
    out.ajouterDouble('temp', this.eres[1]);
    out.ajouterString('comment', this.eres2[0]);
    
    this.t = Math.pow(1. / this.eres[2] / Math.pow(2. * this.h0 * this.h0 * this.Omega_rad0, 0.25) / this.a0, 2.);
    out.ajouterDouble('t', this.t);
    out.ajouterDouble('temp', this.eres[2]);
    out.ajouterString('comment', this.eres2[1]);

    this.t = Math.pow(1. / this.eres[3] / Math.pow(2. * this.h0 * this.h0 * this.Omega_rad0, 0.25) / this.a0, 2.);
    out.ajouterDouble('t', this.t);
    out.ajouterDouble('temp', this.eres[3]);
    out.ajouterString('comment', this.eres2[2]);

    this.t = Math.pow(1. / this.eres[4] / Math.pow(2. * this.h0 * this.h0 * this.Omega_rad0, 0.25) / this.a0, 2.);
    out.ajouterDouble('t', this.t);
    out.ajouterDouble('temp', this.eres[4]);
    out.ajouterString('comment', this.eres2[3]);
    
    out.ajouterDouble('t', 5.391e-44);
    out.ajouterDouble('temp', this.eres[5]);
    out.ajouterString('comment', this.eres2[4]);
    
    out.ajouterDouble('t', 0);
    out.ajouterString('comment', this.eres2[5]);
    
    return out;
}

//initialisation des paramètres d'entrée
AgeUnivers.prototype.initParamIn = function(lin) {
    
    this.h0 = lin.lireDouble('h0');
    this.Omega_m0 = lin.lireDouble('omega-m0');
    this.Omega_l0 = lin.lireDouble('omega-l0');
    this.Omega_rad0 = lin.lireDouble('omega-rad0');
    this.T0 = this.Omega_rad0;
    this.Omega_rad0 = 4.1534e-5 / this.h0 / this.h0 * 100 * 100 * Math.pow((this.Omega_rad0 / 2.725), 4);

    this.a = this.a0;
    this.t = 0;

    this.h0 = this.h0 * 1.e3 / this.mpc;
    this.Omega_k0 = this.Omega_rad0 + this.Omega_m0 + this.Omega_l0 - 1.;

    this.eres = new Array(6);
    this.eres[0] = 1; //Aujourd'hui
    this.eres[1] = 1e3; // CMB
    this.eres[2] = 1e10; // Ere Leptonique
    this.eres[3] = 1e12; // Ere Hadronique
    this.eres[4] = 1e28; // Grande Unification
    this.eres[5] = 1e32; // Temps de Planck

    this.eres2 = new Array(6);
    this.eres2[0] = "Ere Stellaire";
    this.eres2[1] = "Ere Radiative";
    this.eres2[2] = "Ere Leptonique";
    this.eres2[3] = "Ere Hadronique";
    this.eres2[4] = "Grande Unification";
    this.eres2[5] = "Ere de Planck";
}

//calcul du taux d'expansion en fonction de a
AgeUnivers.prototype.hubble = function(scale) {
    var h;
    h = this.Omega_m0 * Math.pow(this.a0 / scale, 3.)
        + this.Omega_rad0 * Math.pow(this.a0 / scale, 4.)
        - this.Omega_k0 * Math.pow(this.a0 / scale, 2.)
        + this.Omega_l0;
    h = this.h0 * Math.sqrt(h);
    return(h);
}

AgeUnivers.prototype.rombint = function(func, x, y, tol) {
    //  Rombint returns the integral from a to b of f(x)dx using Romberg 
    // integration.
    //  The method converges provided that f(x) is continuous in (a,b).  
    //  The function f must be double precision and must be declared external 
    //  in the calling routine.  
    //  tol indicates the desired relative accuracy in the integral.
    var step, gmax, error, g0, fourj, g1, nint, i, jmax, maxiter, maxj, g, k, j;
    maxiter = 20;
    maxj = 5;
    g = new Array(maxj + 2);
    this.failed = false;
    
    step = 0.5 * (y - x);
    gmax = step / y / func.apply(this, [y]) * this.gyr;
    gmax += step / x / func.apply(this, [x]) * this.gyr;
    g[1] = gmax;
    nint = 1;
    error = 1.e20;
    i = 0;
    g0 = 0;
    jmax = 0;
    while ( !(i > maxiter || (i > 5 && Math.abs(error) < tol)) ) {
        i++;
        //	Calculate next trapezoidal rule approximation to integral.
        g0 = 0;
        for (k=0; k < nint; k++) {
            g0 = g0 + 1. / func.apply(this, [x + (k + k - 1) * step]) / ( x + (k + k - 1) * step) * this.gyr;
        }
        g0 = 0.5 * g[1] + step * g0;
        step = 0.5 * step;
        nint = nint + nint;
        jmax = Math.min(i, maxj);
        fourj = 1;
        for (j=0; j<jmax; j++) {
            //	Use Richardson extrapolation.
            fourj = 4. * fourj;
            g1 = g0 + (g0 - g[j]) / (fourj - 1.);
            g[j] = g0;
            g0 = g1;
        }
        if (Math.abs(g0) > tol) {
            error = 1. - gmax / g0;
        } else {
            error = gmax;
        }
        gmax = g0;
        g[jmax+1] = g0;
    }
    //integral=da/x/h*this.gyr;
    if (i > maxiter) { // & Math.abs(error)>tol) {
        //console.log("Rombint failed to converge " + jmax);
        this.failed = true;
    }
    return(g0);
}
